function texture = nd_texture(I,mask,Nbin)

[regionWidth, regionHeight] = size(I);
		
Ibin = ceil(mat2gray(I)*Nbin); % Transform the Image in a binned gray image (from 1 to nbin)
zeroIdx = find(Ibin == 0); % Looks for 0 in Ibin
Ibin(zeroIdx) = 1; % If there is any 0, assign it a 1. By ceiling there shouldn't be any 0

% GLCM
delta = 1; % Step of the GLCM

%angle -> 0 ; (delta, 0) % Looking for horizontal relationship
pixelCounter_H=0;
glcm_H = zeros(Nbin);
		
for x=1:regionWidth-delta
    for y=1:regionHeight
        if mask(x,y) == 1 && mask(x+delta,y) == 1
            a = Ibin(x,y);
            b = Ibin(x+delta,y);
            glcm_H(a,b) = glcm_H(a,b) + 1;
            glcm_H(b,a) = glcm_H(b,a) + 1;
            pixelCounter_H = pixelCounter_H + 2;
        end
    end
end
glcm_H = glcm_H ./ pixelCounter_H;
		
% angle -> 45 ; (delta, -delta)
pixelCounter_RD=0;
glcm_RD = zeros(Nbin);
		
for x=1:regionWidth-delta
    for y=1+delta:regionHeight
        if mask(x,y) == 1 && mask(x+delta,y-delta) == 1
            a = Ibin(x,y);
            b = Ibin(x+delta,y-delta);
            glcm_RD(a,b) = glcm_RD(a,b) + 1;
            glcm_RD(b,a) = glcm_RD(b,a) + 1;
            pixelCounter_RD = pixelCounter_RD + 2;
        end
    end
end
glcm_RD = glcm_RD ./ pixelCounter_RD;

% angle -> 90 ; (0, -delta)
pixelCounter_V=0;
glcm_V = zeros(Nbin);
		
for x=1:regionWidth
    for y=1+delta:regionHeight
        if mask(x,y) == 1 && mask(x,y-delta) == 1
            a = Ibin(x,y);
            b = Ibin(x,y-delta);
            glcm_V(a,b) = glcm_V(a,b) + 1;
            glcm_V(b,a) = glcm_V(b,a) + 1;
            pixelCounter_V = pixelCounter_V + 2;
        end
    end
end
glcm_V = glcm_V ./ pixelCounter_V;

% angle -> 135 ; (-delta, -delta)
pixelCounter_LD=0;
glcm_LD = zeros(Nbin);
		
for x=1+delta:regionWidth
    for y=1+delta:regionHeight
        if mask(x,y) == 1 && mask(x-delta,y-delta) == 1
            a = Ibin(x,y);
            b = Ibin(x-delta,y-delta);
            glcm_LD(a,b) = glcm_LD(a,b) + 1;
            glcm_LD(b,a) = glcm_LD(b,a) + 1;
            pixelCounter_LD = pixelCounter_LD + 2;
        end
    end
end
glcm_LD = glcm_LD ./ pixelCounter_LD;



P = (glcm_H + glcm_RD + glcm_V + glcm_LD)/4;

pX = sum(P,2);
pY = sum(P,1);

muX = 0; muY = 0;
for n=1:Nbin
    muX = muX + n*pX(n);
    muY = muY + n*pY(n);
end

sigmaX = 0; sigmaY = 0;
for n=1:Nbin
    sigmaX = sigmaX + (n-muX)^2*pX(n);
    sigmaY = sigmaY + (n-muY)^2*pY(n);
end
sigmaX = sqrt(sigmaX);
sigmaY = sqrt(sigmaY);

pXplusY = zeros(1,2*Nbin-1); pXminusY = zeros(1,Nbin);

for n=1:Nbin
    for m=1:Nbin
        pXplusY(n+m-1)= pXplusY(n+m-1) + P(n,m);
        pXminusY(abs(n-m)+1) = pXminusY(abs(n-m)+1) + P(n,m); 
    end
end
		
HXY1 = 0; HXY2 = 0;
for n=1:Nbin
    for m=1:Nbin
        if (pX(n)*pY(m)) > 0
            HXY1 = HXY1 - P(n,m)*log(pX(n)*pY(m));
            HXY2 = HXY2 - pX(n)*pY(m)*log(pX(n)*pY(m));
        end
    end
end

HX = 0; HY = 0;
for n=1:Nbin
    if(pX(n) ~= 0) HX = HX - pX(n)*log(pX(n)); end
    if(pY(n) ~= 0) HY = HY - pY(n)*log(pY(n)); end
end
    
% Haralick features
		
%HA1: Angular second moment
ha1 = sum(sum(P.^2));
		
%HA2: Contrast
ha2 = 0;
for k=0:Nbin-1
    temp = 0;
    for n=1:Nbin
        for m=1:Nbin
            if(abs(n-m)==k) temp = temp + P(n,m); end
        end
    end
    ha2 = ha2 + k^2*temp;
end
	
%HA3: Correlation
temp = 0;
for n=1:Nbin
    for m=1:Nbin
        temp = temp + n*m*P(n,m);					
    end
end
ha3 = (temp - muX*muY)/(sigmaX*sigmaY);

%HA4: Sum of squares: Variation
ha4 = 0;
mu = (muX+muY)/2;
for n=1:Nbin
    for m=1:Nbin
        ha4 = ha4 + (n-mu)^2*P(n,m);					
    end
end

% HA5: Inverse difference moment
ha5 = 0;
for n=1:Nbin
    for m=1:Nbin
        ha5 = ha5 + 1/(1+(n-m)^2)*P(n,m);					
    end
end

% HA6: Sum average
ha6 = 0;
for k=2:2*Nbin
	ha6 = ha6 + k*pXplusY(k-1);
end

% HA7: Sum variance
ha7 = 0;
for k=2:2*Nbin
	ha7 = ha7 + (k-ha6)^2*pXplusY(k-1);
end

% HA8: Sum entropy
ha8 = 0;
for k=2:2*Nbin
    if(pXplusY(k-1) > 0) ha8 = ha8 -  pXplusY(k-1)*log(pXplusY(k-1)); end
end

% HA9: Entropy
ha9 = 0;
for n=1:Nbin
    for m=1:Nbin
        if(P(n,m) ~= 0) ha9 = ha9 - P(n,m)*log(P(n,m)); end
    end
end

% HA10: Difference variance
diffAverage = 0;
for k=1:Nbin
    diffAverage = diffAverage + (k-1)*pXminusY(k);
end
ha10 = 0;
for k=1:Nbin
    ha10 = ha10 + (k-1-diffAverage)^2*pXminusY(k);
end

% HA11: Difference entropy
ha11 = 0;
for k=1:Nbin
    if(pXminusY(k) > 0) ha11 = ha11 - pXminusY(k)*log(pXminusY(k)); end
end

% HA12: Information - measure of correlation 1
ha12 = (ha9-HXY1)/max(HX, HY);

% HA13: Information - measure of correlation 2
ha13 = sqrt(1-exp(-2*(HXY2-ha9)));

% Concatenate results
texture = [ha1 ha2 ha3 ha4 ha5 ha6 ha7 ha8 ha9 ha10 ha11 ha12 ha13];

end